//
//  Compat.hpp
//  DDJRBUSBAudio (Kernel Extension)
//
//

#ifndef Compat_hpp
#define Compat_hpp

#include <IOKit/usb/IOUSBHostInterface.h>
#include <IOKit/usb/USB.h>

class IOUSBCompat {
public:
    IOUSBHostDevice* GetDevice( IOUSBHostInterface* hif );
    IOUSBHostPipe* FindNextPipe( IOUSBHostInterface* hif,  IOUSBHostPipe* pipe, IOUSBFindEndpointRequest* epreq );
    IOReturn Read( IOUSBHostPipe* pipe, IOMemoryDescriptor* buffer, UInt32 noDataTimeout, UInt32 completionTimeout, IOByteCount reqCount, IOUSBHostCompletion *completion );
    IOReturn Read( IOUSBHostPipe* pipe, IOMemoryDescriptor * buffer, UInt64 frameStart, UInt32 numFrames, IOUSBHostIsochronousFrame *pFrames, IOUSBHostIsochronousCompletion *completion, UInt32 updateFrequency = 0);
    IOReturn Write( IOUSBHostPipe* pipe, IOMemoryDescriptor* buffer, UInt64 frameStart, UInt32 numFrames, IOUSBHostIsochronousFrame *pFrames, IOUSBHostIsochronousCompletion *completion, UInt32 /*updateFrequency*/ );
    IOReturn SetPipePolicy( IOUSBHostPipe* pipe, UInt16 maxPacketSize );
    IOReturn ResetDevice( IOUSBHostInterface* hif );
    IOReturn GetConfiguration ( IOUSBHostInterface* hif, UInt8* currentConfigValue );
    IOReturn GetLowLatencyOptionsAndPhysicalMask (IOOptionBits* usbOptions, mach_vm_address_t* physicalMask);
    IOReturn GetStringDescriptor( IOUSBHostDevice* device, UInt8 index, char* stringBuffer, size_t stringLength, UInt16 lang = StandardUSB::kLanguageIDEnglishUS );
    IOReturn GetStringDescriptor( IOUSBHostInterface* hif, UInt8 index, char* stringBuffer, size_t stringLength, UInt16 lang = StandardUSB::kLanguageIDEnglishUS );
    IOReturn DeviceRequest( IOUSBHostInterface* hif, IOUSBDevRequest* request, IOUSBCompletion* completion = NULL );
    IOReturn DeviceRequest( IOUSBHostInterface* hif, IOUSBDevRequestDesc* request, IOUSBCompletion* completion );

};

inline void debugIOLogx( const char* file, int line, const char *format, ... )
{
    va_list ap;
    char str[512];
    
    va_start( ap, format );
    
    int offset = (signed)strlen(file) - 30;
    if (offset < 0)
        offset = 0;
    vsnprintf(str, sizeof(str), format, ap);

    kprintf( "shiba: %30s %4d: %s\n", file + offset, line, str );
    //IOSleep( 20/*sleepTime*/ );
    
    va_end( ap );
}

#endif /* Compat_hpp */
